# Rainer Walke, MPI Rostock, 17-Sep-2009
# Technical Report: Piecewise constant baseline hazard data simulation
rm(list = ls())

# SVN identification
# This links the results with the source code.
svn <- unlist(strsplit("$Id: PiecewiseSimulation.R 16 2009-09-17 06:32:43Z walke $"," "))
note <- paste(svn[2],svn[3])

# motivation: orginal data analysis
age <-factor(c(1,1,2,2,3,3,4,4))
period <-factor(c(1,2,1,2,1,2,1,2))
occ <-c(19,10,70,23,134,69,57,27)
expo <-c(1072.5,339,3427,1075,20063,5991,21593,5941)
umea <- data.frame(age, period, occ, expo)
rm(age, period, occ, expo)

umea1 <- glm(occ ~ age + period + offset(log(expo)), family = poisson(link="log"), data = umea)
summary(umea1)
umea1$null.deviance - umea1$deviance
cbind(coef(umea1),confint.default(umea1))
cbind(exp(coef(umea1)), exp(confint.default(umea1)))


# prepare the simulation
library(survival)

# set a fixed starting point for the pseudo-random numbers
set.seed(3456998)

pcbhsim <- function(){
# set the start population size for both groups
  number1 <- 1082
  number2 <- 344
# set the time points
 	TAU <- c(0, 1, 4.33, 26, 52)
	DT = TAU[2:5] - TAU[1:4]
# set the absolute risk
	h0 <- exp(-4.01)
# set the relative risks
	G <- c(1, 1.01, 0.381, 0.150)
# set the relative risks for the population groups
	P <- c(1, 1.54)
# create a helping matrix 
  LD <- matrix(0,nrow=5, ncol=4)
  LD[lower.tri(LD)]<-1

# start with population group 1
  LS <- log(1-runif(number1))
  GP <- P[1]*G
# determine the ln(S) for all TAU
	LSM <- -h0 * as.vector(LD %*% (GP * DT))
# find the appropriate time interval
	t1 <- rep(NA,number1)
		for (i in 1:4) {
	t1 <- ifelse(LSM[i]>=LS & LS>LSM[i+1], TAU[i] + (LSM[i] - LS)/h0/GP[i], t1)
  }
# end of population group1

# start with population group 2
  LS <- log(1-runif(number2))
  GP <- P[2]*G
# determine the ln(S) for all TAU
	LSM <- -h0 * as.vector(LD %*% (GP * DT))
# find the appropriate time interval
	t2 <- rep(NA,number1)
	for (i in 1:4) {
	t2 <- ifelse(LSM[i]>=LS & LS>LSM[i+1], TAU[i] + (LSM[i] - LS)/h0/GP[i], t2)
  }
# end of population group 2

# combine both populations
  sim.data <- data.frame( rbind(cbind(t=t1,period=1),cbind(t2,2)))
  sim.data$occ <- ifelse(is.na(sim.data$t), 0, 1)
  sim.data$t <- ifelse(is.na(sim.data$t), TAU[5], sim.data$t)
  sim.data$id <- row(sim.data)[,1]
# the data set is ready


# compute a proportional hazards Cox model
  sim.cox <- coxph(Surv(t,occ) ~ period, data=sim.data)


# compute a proportional hazards model with piecewise constant baseline hazard
# split the process time
  sim.split <- survSplit(sim.data, cut=c(1, 4.33, 26), end="t", start="t0", event="occ", episode="age")

# +1 just for convenience
  sim.split$age <- factor(sim.split$age+1)
  sim.split$period <- factor(sim.split$period)
  
# compute the hazards using the poisson regression
# it requires to collapse the exposure times and occurences
  sim.split$expo <- sim.split$t - sim.split$t0
  sim.tab <- aggregate(sim.split[c("occ","expo")], by=list(age = sim.split$age, period = sim.split$period), sum)

  sim.poisson <- glm(occ ~ age + period +offset(log(expo)), family = poisson(link="log"), data = sim.tab)

# prepare both results for return
  return(c(exp(coef(sim.poisson)), exp(coef(sim.cox))))

} # end of function pcbhsim

# start the simulation
results <- NULL
reps <- 1000

for (i in 1:reps){
  results <- rbind(results, pcbhsim())
}
# end of simulation

summary(results)

# count the number of trials outside the confidence band
m1 <- exp(confint.default(umea1))
Cabsolut <- ifelse(m1[1,2] < results[,1] | results[,1] < m1[1,1], 1, 0)
Cage2 <- ifelse(m1[2,2] < results[,2] | results[,2] < m1[2,1], 1, 0)
Cage3 <- ifelse(m1[3,2] < results[,3] | results[,3] < m1[3,1], 1, 0)
Cage4 <- ifelse(m1[4,2] < results[,4] | results[,4] < m1[4,1], 1, 0)
Cperiod2 <- ifelse(m1[5,2] < results[,5] | results[,5] < m1[5,1], 1, 0)
CperiodCox <- ifelse(m1[5,2] < results[,6] | results[,6] < m1[5,1], 1, 0)

cbind(table(Cabsolut), table(Cage2), table(Cage3), table(Cage4), table(Cperiod2), table(CperiodCox))


# graph the simulation results
# save default
def.par <- par(no.readonly = TRUE)
layout(matrix(c(1,2,3,4,5,6),2,3,byrow=TRUE), c(1,1,1), c(1,1), FALSE)

# windows(8, 6,  record = TRUE)
hist(results[,2], freq=FALSE, breaks=40, col="gray50", border = "gray50",
  xlim=c(0.5,2.5), xlab="age2", main="", cex.axis=1.3, cex.lab=1.3, mgp=c(2.4,1,0))
  text(1,0.03,note,cex=0.5)

hist(results[,3], freq=FALSE, breaks=40, col="gray50", border = "gray50",
  xlim=c(0.2,0.8), xlab="age3", main="", cex.axis=1.3, cex.lab=1.3, mgp=c(2.4,1,0))
  text(0.4,0.1,note,cex=0.5)

hist(results[,4], freq=FALSE, breaks=40, col="gray50", border = "gray50",
  xlim=c(0.075,0.35), xlab="age4", main="", cex.axis=1.3, cex.lab=1.3, mgp=c(2.4,1,0))
  text(0.15,0.2,note,cex=0.5)

hist(results[,5], freq=FALSE, breaks=40, col="gray50", border = "gray50",
  xlim=c(1,2.1), xlab="period2", main="", cex.axis=1.3, cex.lab=1.3, mgp=c(2.4,1,0))
  text(1.5,0.05,note,cex=0.5)

hist(results[,1], freq=FALSE, breaks=40, col="gray50", border = "gray50",
  xlim=c(0.0075,0.0325), xlab="absolut", main="", cex.axis=1.3, cex.lab=1.3, mgp=c(2.4,1,0))
  text(0.018,2,note,cex=0.5)

hist(results[,6], freq=FALSE, breaks=40, col="gray70", border = "gray70",
  xlim=c(1,2.1), xlab="periodCox", main="", cex.axis=1.3, cex.lab=1.3, mgp=c(2.4,1,0))
  text(1.5,0.05,note,cex=0.5)
  
# reset the graphics device
par(def.par)



#
